package net.sf.wickedshell.domain;

import java.io.IOException;
import java.util.List;

import net.sf.wickedshell.domain.batch.BatchFileDescriptorDao;
import net.sf.wickedshell.domain.command.CommandDescriptorDao;
import net.sf.wickedshell.domain.completion.CompletionDao;
import net.sf.wickedshell.domain.configuration.IShellDescriptorProperties;
import net.sf.wickedshell.domain.configuration.ShellDescriptorPropertiesDao;
import net.sf.wickedshell.domain.style.IColorDescriptor;
import net.sf.wickedshell.domain.style.IFontDescriptor;
import net.sf.wickedshell.domain.style.ShellStyleDao;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.Predicate;
import org.apache.xmlbeans.XmlException;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The activator class controls the plug-in life cycle
 */
public class DomainPlugin extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = "net.sf.wickedshell.domain";
	// The shared instance
	private static DomainPlugin plugin;
	// The document containing the batch files
	private List batchFileDescriptors;
	// The list of static completions
	private List staticCompletions;
	// The command history
	private List commandHistory;
	// The font of the shell
	private IFontDescriptor fontDescriptor;
	// The foreground color
	private IColorDescriptor foregroundColor;
	// The background color
	private IColorDescriptor backgroundColor;
	// The properties for the shell descriptors
	private List shellDescriptorProperties;

	/**
	 * The constructor
	 */
	public DomainPlugin() {
		plugin = this;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		batchFileDescriptors = new BatchFileDescriptorDao().readBatchFileDescriptors();
		staticCompletions = new CompletionDao().readCompletions();
		commandHistory = new CommandDescriptorDao().readCommandHistory();
		ShellStyleDao shellStyleDao = new ShellStyleDao();
		fontDescriptor = shellStyleDao.readFontDescriptor();
		backgroundColor = shellStyleDao.readBackgroundColor();
		foregroundColor = shellStyleDao.readForegroundColor();
		shellDescriptorProperties = new ShellDescriptorPropertiesDao()
				.readShellDescriptorProperties();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		new BatchFileDescriptorDao().writeBatchFileDescriptors(batchFileDescriptors);
		new CompletionDao().writeCompletions(staticCompletions);
		new CommandDescriptorDao().writeCommandHistory(commandHistory);
		new ShellStyleDao().writeShellStyle(fontDescriptor, backgroundColor, foregroundColor);
		new ShellDescriptorPropertiesDao().writeShellDescriptorProperties(shellDescriptorProperties);
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 * 
	 * @return the shared instance
	 */
	public static DomainPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given plug-in relative path.
	 * 
	 * @param path
	 *        the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return AbstractUIPlugin.imageDescriptorFromPlugin("net.sf.wickedshell", path);
	}

	public List getStaticCompletions() {
		return staticCompletions;
	}

	public List getBatchFileDescriptors() {
		return batchFileDescriptors;
	}

	public List getCommandHistory() {
		return commandHistory;
	}

	public IColorDescriptor getBackgroundColor() {
		return backgroundColor;
	}

	public IFontDescriptor getFontDescriptor() {
		return fontDescriptor;
	}

	public IColorDescriptor getForegroundColor() {
		return foregroundColor;
	}

	public void setBackgroundColor(IColorDescriptor backgroundColor) {
		this.backgroundColor = backgroundColor;
	}

	public void setFontDescriptor(IFontDescriptor fontDescriptor) {
		this.fontDescriptor = fontDescriptor;
	}

	public void setForegroundColor(IColorDescriptor foregroundColor) {
		this.foregroundColor = foregroundColor;
	}

	public IShellDescriptorProperties getShellDescriptorProperties(final String shellDescriptorId) {
		IShellDescriptorProperties shellDescriptorProperties = (IShellDescriptorProperties) CollectionUtils
				.find(this.shellDescriptorProperties, new Predicate() {
					/**
					 * @see org.apache.commons.collections.Predicate#evaluate(java.lang.Object)
					 */
					public boolean evaluate(Object object) {
						IShellDescriptorProperties shellDescriptorProperties = (IShellDescriptorProperties) object;
						return shellDescriptorProperties.getShellDescriptorId().equals(shellDescriptorId);
					}
				});
		if (shellDescriptorProperties == null) {
			shellDescriptorProperties = IShellDescriptorProperties.Factory.newInstance(
					shellDescriptorId, new String(), false);
			this.shellDescriptorProperties.add(shellDescriptorProperties);
		}
		return shellDescriptorProperties;
	}

	/**
	 * Resets the <code>StaticCompletionList</code> of static <code>Completions</code> defined by
	 * the guy who wrote this plugin :)
	 */
	public void resetCompletions() {
		try {
			CompletionDao completionDao = new CompletionDao();
			completionDao.writeDefaultCompletions();
			staticCompletions = completionDao.readCompletions();
		}
		catch (IOException exception) {
			exception.printStackTrace();
		}
		catch (XmlException exception) {
			exception.printStackTrace();
		}
	}

	/**
	 * Resets the shell's style defined by the guy who wrote this plugin :)
	 */
	public void resetShellStyle() {
		try {
			ShellStyleDao shellStyleDao = new ShellStyleDao();
			shellStyleDao.writeDefaultShellStyle();
			fontDescriptor = shellStyleDao.readFontDescriptor();
			backgroundColor = shellStyleDao.readBackgroundColor();
			foregroundColor = shellStyleDao.readForegroundColor();
		}
		catch (IOException exception) {
			exception.printStackTrace();
		}
		catch (XmlException exception) {
			exception.printStackTrace();
		}
	}

	/**
	 * Resets the <code>StaticCompletionList</code> of static <code>Completions</code> defined by
	 * the guy who wrote this plugin :)
	 */
	public void resetShellDescriptorProperties() {
		try {
			ShellDescriptorPropertiesDao shellDescriptorPropertiesDao = new ShellDescriptorPropertiesDao();
			shellDescriptorPropertiesDao.writeDefaultShellDescriptorProperties();
			shellDescriptorProperties = shellDescriptorPropertiesDao.readShellDescriptorProperties();
		}
		catch (IOException exception) {
			exception.printStackTrace();
		}
		catch (XmlException exception) {
			exception.printStackTrace();
		}
	}
}
