/*
 * IShellFacade.java
 * 
 * Copyright 2005-2006 Stefan Reichert.
 * All Rights Reserved.
 * 
 * This software is the proprietary information of Stefan Reichert.
 * Use is subject to license terms.
 * 
 */
package net.sf.wickedshell.facade;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import net.sf.wickedshell.ShellID;
import net.sf.wickedshell.facade.descriptor.IShellDescriptor;
import net.sf.wickedshell.facade.history.ICommandHistory;
import net.sf.wickedshell.facade.io.ShellReader;
import net.sf.wickedshell.facade.io.ShellWriter;
import net.sf.wickedshell.listener.IShellListener;
import net.sf.wickedshell.listener.IShellOutputListener;
import net.sf.wickedshell.preferences.PreferenceHelper;

import org.apache.commons.collections.Closure;
import org.apache.commons.collections.CollectionUtils;

/**
 * The <code>IShellFacade</code> is a facade for a selected shell which is configured by an
 * <code>IShellDescriptor</code>.
 * 
 * @author Stefan Reichert
 * @since 17.10.2006
 */
public interface IShellFacade {

	/**
	 * Activates this <code>IShellFacade</code> by starting the underlaying shell and activate the
	 * facade.
	 */
	void activate() throws IOException;

	/**
	 * Returns whether the underlaying shell is active.
	 * 
	 * @return whether the shell is active
	 */
	boolean isActive();

	/**
	 * Deactivates this <code>IShellFacade</code> by terminating the underlaying shell and
	 * deactivate the facade.
	 */
	void deactivate() throws IOException;

	/**
	 * Restart the underlaying shell.
	 */
	void restart();

	/**
	 * Execute the given command.
	 * 
	 * @param command
	 *        The command to execute
	 */
	void executeCommand(String command);

	/**
	 * Returns the current path of the shell.
	 * 
	 * @return the current path of the shell.
	 */
	String getCurrentPath();

	/**
	 * Returns the <code>IShellDescriptor</code> which configures this facade.
	 * 
	 * @return the configuring <code>IShellDescriptor</code>
	 */
	IShellDescriptor getShellDescriptor();

	/**
	 * Returns the history of executed commands.
	 * 
	 * @return the <code>ICommandHistory</code> of this facade
	 */
	ICommandHistory getCommandHistory();

	/**
	 * Adds an <code>IShellOutputListener</code> to the facade. The listener will be notified on
	 * shell output.
	 * 
	 * @param shellOutputListener
	 *        The <code>IShellOutputListener</code> to add
	 */
	void addShellOutputListener(IShellOutputListener shellOutputListener);

	/**
	 * Removes an <code>IShellOutputListener</code> to the facade.
	 * 
	 * @param shellOutputListener
	 *        The <code>IShellOutputListener</code> to remove
	 */
	void removeShellOutputListener(IShellOutputListener shellOutputListener);

	/**
	 * Adds an <code>IShellListener</code> to the facade. The listener will be notified on shell
	 * Command.
	 * 
	 * @param shellListener
	 *        The <code>IShellListener</code> to add
	 */
	void addShellListener(IShellListener shellListener);

	/**
	 * Removes an <code>IShellListener</code> to the facade.
	 * 
	 * @param shellListener
	 *        The <code>IShellListener</code> to remove
	 */
	void removeShellListener(IShellListener shellListener);

	/**
	 * Internal implementation of the <code>IShellFacade</code>.
	 * 
	 * @author Stefan Reichert
	 * @since 18.10.2006
	 */
	class Factory {

		/**
		 * Creates an instance of <code>IShellFacade</code>
		 */
		public static IShellFacade createInstance() {
			return new ShellFacade();
		}
	}

	/**
	 * Internal implementation of the <code>IShellFacade</code>.
	 * 
	 * @author Stefan Reichert
	 * @since 18.10.2006
	 */
	class ShellFacade implements IShellFacade {

		/** The <code>ICommandHistory</code>. */
		private ICommandHistory commandHistory;

		/** The active <code>IShellDescriptor</code>. */
		private IShellDescriptor descriptor;

		/** Flag whether this facade is active. */
		private boolean active;

		/** The <code>Process</code> of the shell. */
		private Process shellProcess;

		/** The <code>ShellReader</code> for input. */
		private ShellReader shellReader;

		/** The <code>ShellReader</code> for errors. */
		private ShellReader shellErrorReader;

		/** The <code>ShellWriter</code> of the shell. */
		private ShellWriter shellWriter;

		/** The <code>ShellListener</code> of the shell. */
		private IShellOutputListener shellOutputListener;

		/** The <code>List</code> of <code>IShellListener</code>s of the shell. */
		private List shellListeners;

		/** The <code>List</code> of <code>IShellOutputListeners</code>s of the shell. */
		private List shellOutputListeners;

		/** The last line of the shell. */
		private String lastLine;

		/**
		 * Constructor for ShellFacade.
		 */
		private ShellFacade() {
			super();
			// private constructor to avoid instantiation
			commandHistory = ICommandHistory.Factory.createInstance();
			shellListeners = new ArrayList();
			shellOutputListeners = new ArrayList();
			lastLine = new String();
			shellOutputListener = new IShellOutputListener() {
				/**
				 * @see net.sf.wickedshell.listener.IShellOutputListener#handleShellOutput(java.lang.String)
				 */
				public void handleShellOutput(final String output) {
					try {
						StringBuffer buffer = new StringBuffer(lastLine);
						buffer.append(output);
						BufferedReader bufferedReader = new BufferedReader(new StringReader(buffer
								.toString()));
						String nextLine = bufferedReader.readLine();
						while (nextLine != null) {
							lastLine = nextLine;
							nextLine = bufferedReader.readLine();
						}
						bufferedReader.close();
						// Delegate to other listeners
						CollectionUtils.forAllDo(shellOutputListeners, new Closure() {
							/**
							 * @see org.apache.commons.collections.Closure#execute(java.lang.Object)
							 */
							public void execute(Object object) {
								IShellOutputListener shellOutputListener = (IShellOutputListener) object;
								shellOutputListener.handleShellOutput(output);
							}
						});
					}
					catch (IOException exception) {
						exception.printStackTrace();
					}
				}

				/**
				 * @see net.sf.wickedshell.listener.ShellOutputAdapter#handleShellErrorOutput(java.lang.String)
				 */
				public void handleShellErrorOutput(final String errorOut) {
					try {
						StringBuffer buffer = new StringBuffer(lastLine);
						buffer.append(errorOut);
						BufferedReader bufferedReader = new BufferedReader(new StringReader(buffer
								.toString()));
						String nextLine = bufferedReader.readLine();
						while (nextLine != null) {
							lastLine = nextLine;
							nextLine = bufferedReader.readLine();
						}
						bufferedReader.close();
						// Delegate to other listeners
						CollectionUtils.forAllDo(shellOutputListeners, new Closure() {
							/**
							 * @see org.apache.commons.collections.Closure#execute(java.lang.Object)
							 */
							public void execute(Object object) {
								IShellOutputListener shellOutputListener = (IShellOutputListener) object;
								shellOutputListener.handleShellErrorOutput(errorOut);
							}
						});
					}
					catch (IOException exception) {
						exception.printStackTrace();
					}
					;
				}
			};
		}

		/**
		 * @see net.sf.wickedshell.facade.IShellFacade#activate()
		 */
		public void activate() throws IOException {
			if (!active) {
				// Active Shell is already set when Shell is restarted
				descriptor = PreferenceHelper.getActiveShellDescriptor();
				shellProcess = Runtime.getRuntime().exec(descriptor.getExecutable(), null,
						new File(PreferenceHelper.getStartupDirectory()));
				Runtime.getRuntime().traceMethodCalls(true);
				Runtime.getRuntime().traceInstructions(true);
				// If process is created manager is active
				active = true;
				// Create ShellReaders
				shellReader = new ShellReader(ShellID.SHELL_READER_NAME, descriptor, shellProcess
						.getInputStream());
				shellErrorReader = new ShellReader(ShellID.SHELL_ERROR_READER_NAME, descriptor,
						shellProcess.getErrorStream(), true);
				// Add the Listener
				shellReader.addShellOutputListener(shellOutputListener);
				shellErrorReader.addShellOutputListener(shellOutputListener);
				// Activate the ShellReaders
				shellReader.activate();
				shellErrorReader.activate();
				// Activate the shellWriter
				shellWriter = new ShellWriter(descriptor, shellProcess.getOutputStream());
				// Notify listeners
				CollectionUtils.forAllDo(shellListeners, new Closure() {
					/**
					 * @see org.apache.commons.collections.Closure#execute(java.lang.Object)
					 */
					public void execute(Object object) {
						IShellListener commandListener = (IShellListener) object;
						commandListener.handleActivation();
					}
				});
			}
		}

		/**
		 * @see net.sf.wickedshell.facade.IShellFacade#deactivate()
		 */
		public void deactivate() throws IOException {
			if (active) {
				shellWriter.deactivate();
				shellReader.deactivate();
				shellErrorReader.deactivate();
				shellProcess.destroy();
				active = false;
				descriptor = null;
				lastLine = new String();
				// Notify listeners
				CollectionUtils.forAllDo(shellListeners, new Closure() {
					/**
					 * @see org.apache.commons.collections.Closure#execute(java.lang.Object)
					 */
					public void execute(Object object) {
						IShellListener commandListener = (IShellListener) object;
						commandListener.handleDeactivation();
					}
				});
			}
		}

		/**
		 * @see net.sf.wickedshell.facade.IShellFacade#executeCommand(java.lang.String)
		 */
		public void executeCommand(final String command) {
			if (active) {
				if (descriptor.isExecutedCommandProvided()) {
					setCurrentCommand(new String());
				}
				else {
					setCurrentCommand(command);
				}
				commandHistory.addCommand(command, descriptor.getId());
				// Notify listeners
				CollectionUtils.forAllDo(shellListeners, new Closure() {
					/**
					 * @see org.apache.commons.collections.Closure#execute(java.lang.Object)
					 */
					public void execute(Object object) {
						IShellListener commandListener = (IShellListener) object;
						commandListener.handleCommandExecution(command);
					}
				});
				// Catch static commands
				if ("exit".equals(command.toLowerCase())) {
					try {
						deactivate();
					}
					catch (IOException exception) {
						exception.printStackTrace();
					}
				}
				else if (!"cls".equals(command.toLowerCase())) {
					shellWriter.writeCommand(command);
				}
			}
			if ("exit".equals(command.toLowerCase())) {
				try {
					deactivate();
				}
				catch (IOException exception) {
					shellErrorReader.handleInput(exception.getMessage());
				}
			}
		}

		/**
		 * Returns the current path of the shell.
		 * 
		 * @return the current path of the shell.
		 */
		public String getCurrentPath() {
			int pathEnd = lastLine.indexOf(descriptor.getPathDelimiter());
			return lastLine.substring(0, pathEnd);
		}

		/**
		 * @see net.sf.wickedshell.facade.IShellFacade#getCommandHistory()
		 */
		public ICommandHistory getCommandHistory() {
			return commandHistory;
		}

		/**
		 * @see net.sf.wickedshell.facade.IShellFacade#getShellDescriptor()
		 */
		public IShellDescriptor getShellDescriptor() {
			return descriptor;
		}

		/**
		 * @see net.sf.wickedshell.facade.IShellFacade#isActive()
		 */
		public boolean isActive() {
			return false;
		}

		/**
		 * @see net.sf.wickedshell.facade.IShellFacade#restart()
		 */
		public void restart() {
			try {
				deactivate();
				// Notify listeners
				CollectionUtils.forAllDo(shellListeners, new Closure() {
					/**
					 * @see org.apache.commons.collections.Closure#execute(java.lang.Object)
					 */
					public void execute(Object object) {
						IShellListener commandListener = (IShellListener) object;
						commandListener.handleRestart();
					}
				});
				activate();
			}
			catch (Exception exception) {
				exception.printStackTrace();
			}
		}

		/**
		 * @see net.sf.wickedshell.facade.IShellFacade#addShellOutputListener(net.sf.wickedshell.listener.IShellOutputListener)
		 */
		public void addShellOutputListener(IShellOutputListener shellOutputListener) {
			shellOutputListeners.add(shellOutputListener);
		}

		/**
		 * @see net.sf.wickedshell.facade.IShellFacade#removeShellOutputListener(net.sf.wickedshell.listener.IShellOutputListener)
		 */
		public void removeShellOutputListener(IShellOutputListener shellOutputListener) {
			shellOutputListeners.remove(shellOutputListener);
		}

		/**
		 * Set the current command of the shell.
		 * 
		 * @param command
		 *        The current command to set
		 */
		private void setCurrentCommand(String command) {
			StringBuffer lastLine = new StringBuffer();
			lastLine.append(getCurrentPath());
			lastLine.append(descriptor.getPathDelimiter());
			lastLine.append(command);
		}

		/**
		 * @see net.sf.wickedshell.facade.IShellFacade#addShellListener(net.sf.wickedshell.listener.IShellListener)
		 */
		public void addShellListener(IShellListener shellListener) {
			shellListeners.add(shellListener);
		}

		/**
		 * @see net.sf.wickedshell.facade.IShellFacade#removeShellListener(net.sf.wickedshell.listener.IShellListener)
		 */
		public void removeShellListener(IShellListener shellListener) {
			shellListeners.remove(shellListener);
		}

	}
}
