/*
 * ShellViewerKeyAdapter.java
 * 
 * Copyright 2005-2006 Stefan Reichert.
 * All Rights Reserved.
 * 
 * This software is the proprietary information of Stefan Reichert.
 * Use is subject to license terms.
 * 
 */
package net.sf.wickedshell.ui.shell.viewer;

import net.sf.wickedshell.facade.history.ICommandHistory;
import net.sf.wickedshell.listener.IShellListener;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Text;

/**
 * @author Stefan Reichert
 * @since 18.10.2006
 */
public class ShellViewerKeyAdapter extends KeyAdapter implements IShellListener {

	/** The history index if no selection ocurred yet. */
	public static final int DEFAULT_SELECTION = -1;

	/** The adapted <code>IShellViewer</code>. */
	private IShellViewer shellViewer;

	/** The index of the history that was requested the latest. */
	private int historyRequestIndex;

	/**
	 * Adapts the given <code>IShellViewer</code>.
	 */
	public static final void adapt(IShellViewer shellViewer) {
		shellViewer.getShellDisplayControl().addKeyListener(new ShellViewerKeyAdapter(shellViewer));
	}

	/**
	 * Constructor for ShellViewerKeyAdapter.
	 */
	private ShellViewerKeyAdapter(IShellViewer shellViewer) {
		super();
		// private constructor to avoid instantiation
		this.shellViewer = shellViewer;
	}

	/**
	 * @see org.eclipse.swt.events.KeyListener#keyPressed(org.eclipse.swt.events.KeyEvent)
	 */
	public void keyPressed(KeyEvent event) {
		if (event.doit) {
//			System.out.println("Character <" + event.character + ">");
//			System.out.println("Widget <" + event.widget + ">");
//			if (event.widget != shellViewer.getShellDisplayControl()) {
//				// Do nothing if event was not issued by the shell's display control
//				event.doit = false;
//			}
			if (event.character == SWT.CR || event.character == SWT.LF) {
				if (shellViewer.isCurrentCaretPositionValid()) {
					// carriage return or newline means enter was pressed --> if within the range of the
					// current command execute current command
					shellViewer.getShellFacade().executeCommand(shellViewer.getCurrentCommand());
				}
				// process linedelimiter in the GUI if the shell doesn't provide it
				if (!shellViewer.getShellFacade().getShellDescriptor().isUILineFeedProvided()) {
					shellViewer.appendToCurrentCommand(Text.DELIMITER);
				}
				historyRequestIndex = DEFAULT_SELECTION;
				event.doit = false;
			}
			else if (event.keyCode == SWT.ARROW_UP) {
				// array up/down within the range of the current command means history access
				if (shellViewer.isCurrentCaretPositionValid()) {
					ICommandHistory commandHistory = shellViewer.getShellFacade().getCommandHistory();
					if (historyRequestIndex >= commandHistory.getSize(shellViewer.getShellFacade()
							.getShellDescriptor().getId())) {
						historyRequestIndex = DEFAULT_SELECTION;
					}
					if ((historyRequestIndex + 1) < commandHistory.getSize(shellViewer.getShellFacade()
							.getShellDescriptor().getId())) {
						historyRequestIndex++;
						shellViewer.setCurrentCommand(shellViewer.getShellFacade().getCommandHistory()
								.getCommand(historyRequestIndex,
										shellViewer.getShellFacade().getShellDescriptor().getId()));
					}
				}
				event.doit = false;
			}
			else if (event.keyCode == SWT.ARROW_DOWN) {
				// array up/down within the range of the current command means history access
				if (shellViewer.isCurrentCaretPositionValid()) {
					ICommandHistory commandHistory = shellViewer.getShellFacade().getCommandHistory();
					if (historyRequestIndex >= commandHistory.getSize(shellViewer.getShellFacade()
							.getShellDescriptor().getId())) {
						historyRequestIndex = DEFAULT_SELECTION;
					}
					if (historyRequestIndex >= 1) {
						historyRequestIndex--;
						shellViewer.setCurrentCommand(commandHistory.getCommand(historyRequestIndex,
								shellViewer.getShellFacade().getShellDescriptor().getId()));
					}
					else {
						historyRequestIndex = DEFAULT_SELECTION;
						shellViewer.setCurrentCommand(new String());
					}
				}

				event.doit = false;
			}
			else if (event.keyCode == SWT.PAGE_UP || event.keyCode == SWT.PAGE_UP) {
				// forbidden keys (vertical navigation)
				event.doit = false;
			}
			else if (event.stateMask == SWT.CTRL && event.keyCode == 'c') {
				// Copy is allowed
			}
			else {
				// everything else only within the range of the current command
				if (shellViewer.isCurrentCaretPositionValid()) {
					// We need special handling of backward-navigation (LEFT, HOME and BACKSPACE)
					if (event.keyCode == SWT.ARROW_LEFT) {
						// Handling of LEFT (keystroke is within the the range of the current command)
						if (event.stateMask == SWT.NONE
								&& shellViewer.getCurrentCaretPosition() == shellViewer
										.getCurrentCommandOffset()) {
							// LEFT only back to the path delimiter
							shellViewer.setCurrentCaretPosition(shellViewer.getCurrentCommandOffset());
							event.doit = false;
						}
						else if (event.stateMask == SWT.SHIFT
								&& shellViewer.getCurrentCaretPosition() == shellViewer
										.getCurrentCommandOffset()) {
							event.doit = false;
						}
						else if (event.stateMask == SWT.CTRL) {
							// Jump left with CTRL-LEFT only back to the path delimiter
							shellViewer.setCurrentCaretPosition(shellViewer.getCurrentCommandOffset());
							event.doit = false;
						}
						else if (event.stateMask == (SWT.CTRL | SWT.SHIFT)) {
							// Jump left with CTRL-SHIFT-LEFT only back to the path delimiter
							int ySelection = shellViewer.getCurrentTextSelection().y;
							shellViewer.setCurrentTextSelection(new Point(shellViewer
									.getCurrentCommandOffset(), ySelection));
							event.doit = false;
						}
						else if (event.stateMask != SWT.NONE && event.stateMask != SWT.SHIFT) {
							// forbidden combination of keys
							event.doit = false;
						}
					}
					else if (event.keyCode == SWT.HOME) {
						// Handling of HOME (keystroke is within the range of the current command)
						if (event.stateMask == SWT.SHIFT) {
							// Jump left with SHIFT-HOME or only back to the path delimiter
							int xSelection = shellViewer.getCurrentTextSelection().x;
							shellViewer.setCurrentTextSelection(new Point(shellViewer
									.getCurrentCommandOffset(), xSelection));
							event.doit = false;
						}
						else if (event.stateMask != SWT.NONE) {
							// forbidden combination of keys (navigation)
							event.doit = false;
						}
						else {
							// Jump left with HOME or only back to the path delimiter
							shellViewer.setCurrentCaretPosition(shellViewer.getCurrentCommandOffset());
							event.doit = false;
						}
					}
					else if (event.character == SWT.BS) {
						// BACKSPACE only back to the path delimiter
						if (shellViewer.getCurrentCaretPosition() == shellViewer
								.getCurrentCommandOffset()) {
							event.doit = shellViewer.getCurrentTextSelectionCount() > 0;
						}
					}
					else if (event.keyCode == SWT.TAB) {
						// forbidden key --> TAB is CA activation
						event.doit = false;
					}
				}
				else if (event.keyCode != SWT.CTRL && shellViewer.isPathDelimiterAvailableInLastLine()) {
					shellViewer.appendToCurrentCommand(shellViewer.getShellFacade().getShellDescriptor()
							.getPathDelimiter());
					shellViewer.appendToCurrentCommand(String.valueOf(event.character));
					shellViewer.setCurrentCaretPosition(shellViewer.getContentLength());
					event.doit = false;
				}
				else {
					// everything else out the range of the current command is forbidden
					event.doit = false;
				}
			}
		}
	}

	/**
	 * @see net.sf.wickedshell.listener.IShellListener#handleActivation()
	 */
	public void handleActivation() {
	}

	/**
	 * @see net.sf.wickedshell.listener.IShellListener#handleCommandExecution(java.lang.String)
	 */
	public void handleCommandExecution(String command) {
		historyRequestIndex = DEFAULT_SELECTION;
	}

	/**
	 * @see net.sf.wickedshell.listener.IShellListener#handleDeactivation()
	 */
	public void handleDeactivation() {
	}

	/**
	 * @see net.sf.wickedshell.listener.IShellListener#handleRestart()
	 */
	public void handleRestart() {
	}

}
